@@ -1,19 +1,10 @@
 #![cfg(test)]
 #![allow(non_upper_case_globals)]
 #![allow(dead_code)]
+
 extern crate brotli_decompressor;
 extern crate core;
-#[allow(unused_imports)]
-use super::alloc_no_stdlib::{Allocator, SliceWrapper, SliceWrapperMut};
-use super::brotli::BrotliResult;
-use super::brotli::BrotliState;
-#[cfg(feature = "std")]
-use super::brotli::{CompressorReader, CompressorWriter};
-#[cfg(feature = "std")]
-use super::brotli_decompressor::{Decompressor, DecompressorWriter};
-use super::HeapAllocator;
-use super::Rebox;
-use brotli::BrotliDecompressStream;
+
 use core::cmp::min;
 use std::io;
 #[cfg(feature = "std")]
@@ -22,6 +13,16 @@ use std::time::Duration;
 #[cfg(not(feature = "disable-timer"))]
 use std::time::SystemTime;
 
+use brotli::BrotliDecompressStream;
+
+use super::alloc_no_stdlib::{Allocator, SliceWrapper, SliceWrapperMut};
+use super::brotli::{BrotliResult, BrotliState};
+#[cfg(feature = "std")]
+use super::brotli::{CompressorReader, CompressorWriter};
+#[cfg(feature = "std")]
+use super::brotli_decompressor::{Decompressor, DecompressorWriter};
+use super::{HeapAllocator, Rebox};
+
 #[cfg(feature = "benchmark")]
 extern crate test;
 #[cfg(feature = "benchmark")]
@@ -32,8 +33,9 @@ pub struct Buffer {
     read_offset: usize,
 }
 
+#[derive(Debug)]
 pub struct UnlimitedBuffer {
-    data: Vec<u8>,
+    pub data: Vec<u8>,
     read_offset: usize,
 }
 
